package gov.grants.apply.applicant.v1;

import java.util.List;

import gov.grants.apply.webservices.applicantintegrationservices_v1.ApplicantIntegrationPortType;
import gov.grants.apply.webservices.applicantintegrationservices_v1.ApplicationInformationType;
import gov.grants.apply.webservices.applicantintegrationservices_v1.GetApplicationListRequest;
import gov.grants.apply.webservices.applicantintegrationservices_v1.GetApplicationListRequest.ApplicationFilter;
import gov.grants.apply.webservices.applicantintegrationservices_v1.GetApplicationListResponse;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.log4j.Logger;


public class GetApplicationListClient extends BaseApplicantClient {
	
	private static final String CLASSNAME = GetApplicationListClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	/**
	 * @param args
	 */
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			GetApplicationListClient client = new GetApplicationListClient();
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nException: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	public void makeServiceCall() throws Exception {
		try {
			String type = getArgMap().get( Globals.FILTER_CMD_LINE_KEY );
			log.debug( "Filter: " + type );
			String value = getArgMap().get( Globals.FILTER_VALUE_CMD_LINE_KEY );
			log.debug( "Filter Value: " + value );
			// No need to assert type & value since they could be null.
			ApplicantIntegrationPortType port = getApplicantPort();
			GetApplicationListRequest request = new GetApplicationListRequest();
			ApplicationFilter f = new ApplicationFilter();
			f.setFilter(type);
			f.setFilterValue(value);
			request.getApplicationFilter().add(f);
			
			GetApplicationListResponse response = port.getApplicationList(request);

			log.debug( "available appplication count: " + response.getAvailableApplicationNumber() );
			List <ApplicationInformationType> list = response.getApplicationInformation();
			
			printList( list );
			log.debug( "Total Available Application Count: " + response.getAvailableApplicationNumber());
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall
	
	public void printList( List<ApplicationInformationType> list ) {
		if ( list == null || list.size() == 0 ) {
			log.debug( "Application list is empty." );
				return;
		}// if
	         
		log.debug( "Received " + list.size() + " applications in list" );
		StringBuilder sb = new StringBuilder().append( "\n" );
//		for (int i = 0; list != null && i < list.length; i++)
		for ( ApplicationInformationType info : list ) {
			log.debug( "\n\n" );
			log.debug( ToStringBuilder.reflectionToString( info ) );
			log.debug( "\n\n" );
		}// for-each
	}// printList
}
