package gov.grants.apply.applicant.v1;

import gov.grants.apply.webservices.applicantintegrationservices_v1.ApplicantIntegrationPortType;
import gov.grants.apply.webservices.applicantintegrationservices_v1.GetOpportunityListRequest;
import gov.grants.apply.webservices.applicantintegrationservices_v1.GetOpportunityListResponse;
import gov.grants.apply.webservices.applicantintegrationservices_v1.OpportunityInformationType;

import java.util.List;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.log4j.Logger;

public class GetOpportunityListClient extends BaseApplicantClient {

	private static final String CLASSNAME = GetOpportunityListClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	/**
	 * @param args
	 */
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			GetOpportunityListClient client = new GetOpportunityListClient();
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nException: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String cfda = getArgMap().get( Globals.CFDA_CMD_LINE_KEY );
			log.debug( "cfda: " + cfda );
			
			String oppId = getArgMap().get( Globals.OPP_ID_CMD_LINE_KEY );
			log.debug( "opp ID: " + oppId );
			
			String compId = getArgMap().get( Globals.COMP_ID_CMD_LINE_KEY );
			log.debug( "comp ID: " + compId );
			
			ApplicantIntegrationPortType port = getApplicantPort();
			GetOpportunityListRequest request = new GetOpportunityListRequest();
			
			request.setOpportunityID( oppId );
			request.setCFDANumber( cfda );
			request.setCompetitionID( compId );
			
			GetOpportunityListResponse response = port.getOpportunityList( request );
			List<OpportunityInformationType> list = response.getOpportunityInformation();
			
			printList( list );
			log.debug( "list count: " + list.size() );
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall

	
	
	public void printList( List<OpportunityInformationType> list ) {
		if ( list == null || list.size() == 0 ) {
			log.debug( "Application list is empty." );
				return;
		}// if
	         
		log.debug( "Received " + list.size() + " applications in list" );
		StringBuilder sb = new StringBuilder().append( "\n" );
//		for (int i = 0; list != null && i < list.length; i++)
		int count = 0;
		for ( OpportunityInformationType info : list ) {
			sb.append( "\n" );
			log.debug( ToStringBuilder.reflectionToString( info ) );
			/*
			 * "opportunityID", "opportunityTitle", "openingDate", "closingDate", "cfdaNumber", 
			 * "competitionID", "schemaURL", "instructionURL"
			 */
			sb.append( "\nOpportunity ID: " ).append( info.getOpportunityID() );
			sb.append( "\nOpportunity Title: " ).append( info.getOpportunityTitle() );
			sb.append( "\nOpening Date: " );
			if ( info.getOpeningDate() != null ) {
				sb.append( formatDate( info.getOpeningDate().toGregorianCalendar().getTime(), MMddyyyy ) );
			}// if
			
			sb.append( "\nClosing Date: " );
			if ( info.getClosingDate() != null ) {
				sb.append( formatDate( info.getClosingDate().toGregorianCalendar().getTime(), MMddyyyy ) );
			}// if
			
			sb.append( "\nCFDA Number: " ).append( info.getCFDANumber() );
			sb.append( "\nCompetition ID: " ).append( info.getCompetitionID() );
			sb.append( "\nSchema URL: " ).append( info.getSchemaURL() );
			sb.append( "\nInstruction URL: " ).append( info.getInstructionURL() );
			sb.append( "\n\n" );
			
			log.debug( sb );
			sb.setLength( 0 );
			count++;
		}// for-each
		
		log.debug( "count: " + count );
		
	}// printList
	
	

}
