package gov.grants.apply.applicant.v1;

import gov.grants.apply.applicant.v1.util.AttachmentUtil;
import gov.grants.apply.webservices.applicantintegrationservices_v1.ApplicantIntegrationPortType;
import gov.grants.apply.webservices.applicantintegrationservices_v1.SubmitApplicationRequest;
import gov.grants.apply.webservices.applicantintegrationservices_v1.SubmitApplicationResponse;
import gov.grants.commons.util.FileUtil;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.activation.DataHandler;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.ximpleware.VTDNav;

public class SubmitApplicationClient extends BaseApplicantClient {

	private static final String CLASSNAME = SubmitApplicationClient.class.getSimpleName();
	private static final Logger log = Logger.getLogger( CLASSNAME );
	
	
	/**
	 * @param args
	 */
	public static void main( String[] args ) {
		log.debug( "Begin " + CLASSNAME );
		
		try {
			SubmitApplicationClient client = new SubmitApplicationClient();
			
			log.debug( "args length: " + args.length );
			client.init( args );
			
			client.makeServiceCall();
			log.debug( "\n\nSUCCESS: " + CLASSNAME + " successfully completed" );

		} catch ( Exception e ) {
			log.error( "\n\nException: " + e.getMessage() );
		}// try-catch
		
	}// main
	
	
	public void makeServiceCall() throws Exception {
		try {
			String configXml = getArgMap().get( Globals.CONFIG_XML_CMD_LINE_KEY );
			loadConfigXmlDoc( configXml );
			configVN.toElement( VTDNav.FIRST_CHILD, CLASSNAME );// navigate to client element
			
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.SUBMISSION_DIR_ELEMENT_NAME );
			String submissionDir = configVN.toNormalizedString( configVN.getText() );
			log.debug( "submissionDir: " + submissionDir );
			
			configVN.toElement( VTDNav.PARENT );
			configVN.toElement( VTDNav.FIRST_CHILD, Globals.SUBMISSION_XML_ELEMENT_NAME );
			String xmlFileName = configVN.toNormalizedString( configVN.getText() );
			log.debug( "xmlFileName: " + xmlFileName );
			
			if ( StringUtils.isBlank( submissionDir ) || StringUtils.isBlank( xmlFileName ) ) {
				String s = "submission-dir and submission-xml elements in the config xml cannot be blank";
				log.error( s );
				throw new Exception( s );
			}// if
			
			byte[] fileBytes = null;
			String fileName = null;
			String cid = null;
			String contentType = null;
			DataHandler dh = null;
			Map<String, DataHandler> attachmentMap = new HashMap<String, DataHandler>();
			log.debug( "load attachment(s)" );
			File att = null;
			
			while ( configVN.toElement( VTDNav.NEXT_SIBLING, Globals.ATTACHMENT_ELEMENT_NAME ) ) {
				
				configVN.toElement( VTDNav.FIRST_CHILD, Globals.FILE_NAME_ELEMENT_NAME );
				fileName = configVN.toNormalizedString2( configVN.getText() );
				log.debug( "attachment file name: " + fileName );
//				log.debug( configVN.toNormalizedString2( configVN.getCurrentIndex() ) + ": " + fileName );
				
				att = new File( submissionDir + fileName );
				log.debug( "try to load: " + att.getCanonicalPath() );
				if ( !att.exists() ) {
					log.warn( "File does not exist: " + att.getCanonicalPath() );
					att = new File( fileName );
					log.debug( "try to load: " + att.getCanonicalPath() );
					if ( !att.exists() ) {
						log.error( "File does not exist: " + att.getCanonicalPath() );
						throw new Exception( "File does not exist: " + att.getCanonicalPath() );
					}// if
				}// if
				
				contentType = new javax.activation.MimetypesFileTypeMap().getContentType( att );
				log.debug( "content type: " + contentType );
				fileBytes = FileUtil.getBytesFromFile( att );
				log.debug( "file bytes: " + fileBytes );
				
				configVN.toElement( VTDNav.PARENT );
				configVN.toElement( VTDNav.FIRST_CHILD, Globals.CID_ELEMENT_NAME );
				cid = configVN.toRawString( configVN.getText() );
				log.debug( "cid: " + cid );
//				log.debug( configVN.toNormalizedString2( configVN.getCurrentIndex() ) + ": " + cid );
				
				dh = AttachmentUtil.createDataHandler( contentType, cid, fileBytes );
//				dh = AttachmentUtil.createStreamingDataHandler( fileBytes.length );
				attachmentMap.put( cid, dh );
				
				configVN.toElement( VTDNav.PARENT );
			}// while
			
//			ApplicantIntegrationPortType port = super.getStreamingApplicantPort();
			ApplicantIntegrationPortType port = super.getApplicantPort();
			
			if ( attachmentMap.size() > 0 ) {
				log.debug( "add outbound attachments" );
				super.addOutboundAttachmentsToMsgCtx( port, attachmentMap );
			}// if
			
			log.debug( "load submission xml" );
			fileBytes = FileUtil.readFile( submissionDir + xmlFileName );
			log.debug( "xml bytes: " + fileBytes.length );
			log.debug( "xml size: " + new String( fileBytes ).length() );
			
			SubmitApplicationRequest request = new SubmitApplicationRequest();
			request.setGrantApplicationXML( new String( fileBytes ) );
			
			SubmitApplicationResponse response = port.submitApplication( request );
			
			String ggTrackingNum = response.getGrantsGovTrackingNumber();
			String receivedDateTime = formatDate( response.getReceivedDateTime().toGregorianCalendar().getTime(), yyyyMMddHHmmss );
			
			StringBuilder sb = new StringBuilder();
			sb.append( "\n\nGG Tracking Number: " ).append( ggTrackingNum );
			sb.append( "\nReceived Date/Time: " ).append( receivedDateTime );
			sb.append( "\n\n" );
			log.debug( sb );
			
		} catch ( Exception e ) {
			log.error( "Exception: " + e.getMessage() );
			e.printStackTrace();
			throw e;
		}// try-catch
		
	}// makeServiceCall
	

}
