package gov.grants.apply.applicant.v1.handler;


import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.apache.log4j.Logger;

public class ApplicantClientSOAPHandler implements SOAPHandler<SOAPMessageContext> {
	
	private static final Logger log = Logger.getLogger( ApplicantClientSOAPHandler.class.getName() );
	public static final String CONTENT_DISPOSITION_MIME_HEADER = "Content-Disposition";
	
	/* Public Method(s) */
	
	public boolean handleMessage( SOAPMessageContext smc ) {
		
		SOAPMessage msg = smc.getMessage();
//		WSHandlerUtil.displaySOAPMsg( msg );
//		WSHandlerUtil.getSOAPMsgName( msg, AgencyHandler.WITHOUT_NS );
		
		Boolean outbound = ( Boolean ) smc.get( MessageContext.MESSAGE_OUTBOUND_PROPERTY );
		log.debug( "outbound: " + outbound );
		if ( outbound ) {
			log.debug( "outbound message -- check for attachments" );
			// key: contentID --- value: DataHandler
			Map<String, DataHandler> attachmentMap = ( Map<String, DataHandler> ) smc.get( MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS );
			log.debug( "attachment map: " + attachmentMap );
			if ( attachmentMap != null && attachmentMap.size() > 0 ) {
				log.info( "Adding attachments to soap message" );
				AttachmentPart part = null;
				DataHandler attachment = null;
				Set<String> keys = attachmentMap.keySet();
				for ( String key : keys ) {
					log.debug( "Adding attachment for: " + key );
					attachment = attachmentMap.get( key );
					log.debug( "attachment bytes name: " + attachment.getName() );
					part = msg.createAttachmentPart( attachment );
					part.setMimeHeader( CONTENT_DISPOSITION_MIME_HEADER, "attachment; filename=\"" + attachment.getName() + "\"" );
					part.setContentId( key );
					//	MAMER: Adding an attachment Part like this in WL 10.3 results in duplication of file contents.
					//msg.addAttachmentPart( part );
					log.debug( "Attachment successfully added" );
				}// for
				
			}// if
			
		} else {
			log.debug( "inbound message --- check for attachments" );
			// NOTE: no need to process attachments as they are already available in the MessageContext
			log.debug( "check inbound attachments: " + smc.get( MessageContext.INBOUND_MESSAGE_ATTACHMENTS ) );

		}// if-else
		
		
		return true;
	}// handleMessage
	
	
	
	public Set<QName> getHeaders() {
		// Weblogic 10.0 doesn't accept null return value of the SOAPHandler method getHeaders(). 
		// Will throw NullPointerException at startup.  (Apparently is fixed in 10.3)
		return new HashSet<QName>();
	}// getHeaders
	
	
	
	public void close( MessageContext mc ) {
		log.debug( "perform cleanup" );
		System.out.println( "AgencySOAPHandler->perform cleanup" );
	}// close
	
	
	public boolean handleFault( SOAPMessageContext msc ) {
		return true;
	}// handleFault
	

}
