package gov.grants.apply.applicant.v1.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import gov.grants.commons.s2s.util.ByteArrayDataSource;

import javax.activation.DataHandler;
import javax.activation.DataSource;

import org.apache.log4j.Logger;

public class AttachmentUtil {
	
	private static final Logger log = Logger.getLogger( AttachmentUtil.class.getName() );
	
	public static DataSource createDataSource( String contentType, String fileName, byte[] bytes ) {
		DataSource ds = null;
		if ( bytes != null && bytes.length > 0 ) {
			log.debug( "create ByteArrayDataSource for: " + fileName );
			ds = new ByteArrayDataSource( contentType, fileName, bytes );
		}// if
		
		return ds;
		
	}// createDataSource
	
	
	public static DataHandler createDataHandler( String contentType, String fileName, byte[] bytes ) {
		DataSource ds = createDataSource( contentType, fileName, bytes );
		DataHandler dh = null;
		if ( ds != null ) {
			log.debug( "create DataHandler for: " + fileName );
			// TODO add code to handle different encoding types
			dh = new DataHandler( ds );
		}// if
		
		return dh;
		
	}// createDataHandler
	
	
    /**
     * Streaming data handler
     */
    public static DataHandler createStreamingDataHandler( final int total )  {
    	log.debug( "create streaming data handler for file size: " + total );
        return new DataHandler( new DataSource() {
            public InputStream getInputStream() throws IOException {
                return new InputStream() {
                    int i;

                    @Override
                    public int read() throws IOException {
                        return i<total ? 'A'+(i++%26) : -1;
                    }
                };
            }

            public OutputStream getOutputStream() throws IOException {
                return null;
            }

            public String getContentType() {
                return "application/octet-stream";
            }

            public String getName() {
                return "";
            }
        });
    }// getStreamingDataHandler
	
	
	public static DataHandler createDataHandler( String contentType, String fileName, byte[] bytes, String encodingType ) {
		DataSource ds = createDataSource( contentType, fileName, bytes );
		DataHandler dh = null;
		if ( ds != null ) {
			log.debug( "create DataHandler for [" + fileName + "] with encoding type [" + encodingType + "]" );
			// TODO add code to handle different encoding types
			dh = new DataHandler( ds );
		}// if
		
		return dh;
		
	}// createDataHandler
	
	
//	public static void addOutboundAttachmentsToMsgCtx( WebServiceContext wsCtx, Map<String, DataHandler> attachmentMap ) 
//	throws Exception {
//		log.debug( "Add outbound attachments to MessageContext: " + attachmentMap );
//		try {
//			log.debug( "Get handle to message context" + wsCtx.getMessageContext().getClass().getName() );
//			MessageContext msgCtx = wsCtx.getMessageContext();
//			msgCtx.put( MessageContext.OUTBOUND_MESSAGE_ATTACHMENTS , attachmentMap );
//			
//		} catch ( Exception e ) {
//			log.error( "Exception caught adding attachments to MessageContext: " + e.getMessage() );
//			throw e;
//		}// try-catch
//		
//	}// addAttachmentsToMessageContext
	
}
