package gov.grants.apply.applicant.v1.util;


import gov.grants.apply.webservices.applicantIntegrationServices.GetApplicationInfoResponseXmlDocument;
import gov.grants.apply.webservices.applicantIntegrationServices.GetApplicationInfoResponseXmlDocument.GetApplicationInfoResponseXml;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlError;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;

public class XMLBeansUtil {
	
	private static final Logger log = Logger.getLogger( XMLBeansUtil.class.getName() );
	
	
	
	public static XmlOptions getXMLBeansXmlOptions( boolean indent ) {
		// extract XML string
		XmlOptions opts = new XmlOptions();
		opts.setSaveOuter();// remove the <xml-fragment> wrapper element
		if ( indent ) {
			opts.setSavePrettyPrint().setSavePrettyPrintIndent( 4 );
		}// if
		return opts;
	}// getXMLBeansXmlOptions
	
	
	
	public static GetApplicationInfoResponseXml parseGetApplicationInfoResponseXml( String xmlStr, boolean validate ) 
	throws XmlException, Exception {
		GetApplicationInfoResponseXmlDocument doc = null;
		GetApplicationInfoResponseXml xml = null;
		String xmlType = GetApplicationInfoResponseXml.class.getSimpleName();
		
		try {
			log.debug( "parse XML" );
			XmlOptions opts = new XmlOptions();
			opts.setLoadLineNumbers();
			doc = GetApplicationInfoResponseXmlDocument.Factory.parse( xmlStr, opts );
//			log.debug( "validate doc: " + doc );
			
			if ( validate ) {
				validateXml( doc );
			}// if
			
			xml = doc.getGetApplicationInfoResponseXml();
			
		} catch ( XmlException e ) {
			log.error( "XmlException caught parsing " + xmlType + ": " + e.getMessage() );
			throw e;
		} catch ( Exception e ) {
			log.error( "Exception caught validating " + xmlType + ": " + e.getMessage() );
			throw e;
		}// try-catch
		
		log.debug( "successfully parsed " + xmlType );
		return xml;
			
	}// parseGetApplicationInfoResponseXml
	
	
//	public static GetOpportunitiesResponseXml parseGetOpportunitiesResponseXml( String xmlStr, boolean validate ) 
//	throws XmlException, Exception {
//		GetOpportunitiesResponseXmlDocument doc = null;
//		GetOpportunitiesResponseXml xml = null;
//		String xmlType = GetOpportunitiesResponseXml.class.getSimpleName();
//		
//		try {
//			log.debug( "parse XML" );
//			XmlOptions opts = new XmlOptions();
//			opts.setLoadLineNumbers();
//			doc = GetOpportunitiesResponseXmlDocument.Factory.parse( xmlStr, opts );
////			log.debug( "validate doc: " + doc );
//			
//			if ( validate ) {
//				validateXml( doc );
//			}// if
//			
//			xml = doc.getGetOpportunitiesResponseXml();
//			
//		} catch ( XmlException e ) {
//			log.error( "XmlException caught parsing " + xmlType + ": " + e.getMessage() );
//			throw e;
//		} catch ( Exception e ) {
//			log.error( "Exception caught validating " + xmlType + ": " + e.getMessage() );
//			throw e;
//		}// try-catch
//		
//		log.debug( "successfully parsed " + xmlType );
//		return xml;
//			
//	}// parseGetOpportunitiesResponseXml
	
	
//	public static AuthenticateAORResponseXml parseAuthenticateAORResponseXml( String xmlStr, boolean validate ) 
//	throws XmlException, Exception {
//		AuthenticateAORResponseXmlDocument doc = null;
//		AuthenticateAORResponseXml xml = null;
//		String xmlType = AuthenticateAORResponseXml.class.getSimpleName();
//		
//		try {
//			log.debug( "parse XML" );
//			XmlOptions opts = new XmlOptions();
//			opts.setLoadLineNumbers();
//			doc = AuthenticateAORResponseXmlDocument.Factory.parse( xmlStr, opts );
////			log.debug( "validate doc: " + doc );
//			
//			if ( validate ) {
//				validateXml( doc );
//			}// if
//			
//			xml = doc.getAuthenticateAORResponseXml();
//			
//		} catch ( XmlException e ) {
//			log.error( "XmlException caught parsing " + xmlType + ": " + e.getMessage() );
//			throw e;
//		} catch ( Exception e ) {
//			log.error( "Exception caught validating " + xmlType + ": " + e.getMessage() );
//			throw e;
//		}// try-catch
//		
//		log.debug( "successfully parsed " + xmlType );
//		return xml;
//			
//	}// parseAuthenticateAORResponseXml
	
	
	
	/* Protected Method(s) */
	
	
	protected static void validateXml( XmlObject xmlObj ) throws Exception {
		List<XmlError> errors = new ArrayList<XmlError>();
		boolean isValid = validateXml( xmlObj, errors );
		log.debug( "is xml valid: " + isValid );
		if ( errors.size() > 0 ) {
			log.error( "XML validation errors found: " + errors );
			String errorMsg = formatErrors( errors );
			
			throw new Exception( "XMLValidation errors found: " + errorMsg );
		}// if
	}// validateXml
	
	
	protected static boolean validateXml( XmlObject xmlObj, List<XmlError> errors ) throws Exception {
		try {
			// validate XML
//			List<XmlError> errors = new ArrayList<XmlError>();
			XmlOptions opts = new XmlOptions();
			opts.setLoadLineNumbers();// must also be set when parsing document
			opts.setErrorListener( errors );
			boolean valid = xmlObj.validate( opts );
			log.debug( "is valid: " + valid );
			log.debug( "errors: " + errors );
			return valid;
			
		} catch ( Exception e ) {
			log.error( "Exception caught validating XML: " + e.getMessage() );
			throw e;
		}// try-catch
		
	}// validateXml
	
	
	protected static String formatErrors( List<XmlError> errors ) throws Exception {
		StringBuilder sb = new StringBuilder();
		if ( errors == null || errors.isEmpty() ) {
			log.warn( "errors list is empty, returning empty string" );
			return sb.toString();
		}// if
		
		int count = 1;
		try {
			for ( XmlError error : errors ) {
				log.debug( "line: " + error.getLine() );
				log.debug( "error msg: " + error.getMessage() );
				if ( count > 1 ) {
					sb.append( "; " );
				}// if
				
				sb.append( "error (" )
					.append( count )
					.append( ") at " )
					.append( "line " )
					.append( error.getLine() )
					.append( ": " )
					.append( error.getMessage() );
				
				count++;
			}// for
			
		} catch ( Exception e ) {
			log.error( "Exception caught formatting error message: " + e.getMessage() );
			throw new Exception( "Exception caught formatting error message: " + e.getMessage() );
		}// try-catch
		
		log.debug( "returning formatted error message: " + sb );
		return sb.toString();
	}// formatErrors
	
	
}
